/*------------------------------------------------------------------------------

  Copyright (c) 2018-2022 boshra rajaei (bo.rajaei@gmail.com)

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.


  partition.c - This file belongs to ELSDc project (Ellipse and Line Segment
           Detector with continuous validation).
         - It contains the definition of various functions needed in different
           parts of the project.

------------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "polygon.h"
#include "partition.h"
#include "misc.h"

long factorial(int n)
{
  int c;
  long result = 1;

  for (c = 1; c <= n; c++)
    result = result * c;

  return result;
}

//factorial(n)/(factorial(k)*factorial(n-k))
int bin_term(int n, int k)
{
    long result, nom = 1;
    if(k<=n/2)
    {
        for(int i=0;i<k;i++)
            nom *= n-i;
        result = nom/factorial(k);

    }
    else
    {
        for(int i=0;i<n-k;i++)
            nom *= n-i;
        result = nom/factorial(n-k);
    }

    if(result>pow(2,8*sizeof(int)))
        error("generate_partition: very large number of partitions. Should be handled later! ");

    return (int)result;

}

void combination(int r, int index, int data[], int **cp, int *cnt)
{
    // Current combination is ready, print it
    if (index == r) {
        cp[*cnt] = (int*) malloc( r * sizeof(int) );//new int[r];
        for (int j = 0; j < r; j++)
            cp[*cnt][j] = data[j];
        (*cnt)++;
        return;
    }

    data[index] = 0;
    combination(r, index + 1, data, cp, cnt);

    data[index] = 1;
    combination(r, index + 1, data, cp, cnt);

    data[index] = 2;
    combination(r, index + 1, data, cp, cnt);

}

void SubSet(int arr[], int n, int r, int index,
                     int data[], int i, int **cp, int *cnt)//, int *fixed, int fixed_cnt)
{
    // Current combination is ready, print it
    if (index == r) { // && !fixed_cnt
        cp[*cnt] = (int*) malloc( r * sizeof(int) );
        for (int j = 0; j < r; j++)
            cp[*cnt][j] = data[j];
        (*cnt)++;
        return;
    }
    //else if(index==r && fixed_cnt) //we set r cut points but still not meet all fixed cut points
    //    return;

    // When no more elements are there to put in data[]
    if (i >= n)
        return;

    //if(fixed[i])
    //{
        // current is a fixed point and is included
    //    data[index] = arr[i];
    //    SubSet(arr, n, r, index + 1, data, i + 1, cp, cnt, fixed, --fixed_cnt);
    //}
    //else
    //{
        // current is included, put next at next location
        data[index] = arr[i];
        SubSet(arr, n, r, index + 1, data, i + 1, cp, cnt);//, fixed, fixed_cnt);

        // current is excluded, replace it with next
        // (Note that i+1 is passed, but index is not
        // changed)
        SubSet(arr, n, r, index, data, i + 1, cp, cnt);//, fixed, fixed_cnt);
    //}

}

void generate_partition(parList *par, int *parcnt, PolyRect poly, int max_cnt_point)
{
    int d = poly.dim;
    int arr[d-1];
    for(int i=0;i<d-1;i++) arr[i] = i+1;
    //parList *par;
    //par = (parList*) malloc( sizeof(parList) );
    //if( par == NULL ) error("ELSDc: not enough memory.");
    parList *strpntr = par;
    parList *prevpntr = par;
    //int cntL = 0;
    int **cp, **ptype;
    int size_ptype;
    int size_cp;
    int cnt, cnt1;
    (*parcnt) = 0;

    /*int fixed[d-1];
    int fixed_cnt = 0;

    for(int i=0;i<d-1;i++)
        if(!is_aligned( poly.rectlist[i].theta, poly.rectlist[i+1].theta, prec ))
        {
            fixed[i] = 1;
            fixed_cnt++;
        }
        else
        {
            fixed[i] = 0;
        }*/
    int lmt1 = min(max_cnt_point, d);//max(min(max_cnt_point, d), fixed_cnt);
    //int lmt2 = max(6, d-5);


    for(int p=1;p<=lmt1;p++)//=max(1,d-1))//p<=d//(fixed_cnt>0?fixed_cnt:1)
    {

    size_ptype = pow(3,p);
    size_cp = bin_term(d-1,p-1);
    cp = (int**) malloc( size_cp * sizeof(int*) );
    if( cp == NULL ) error("generate_partition: not enough memory.");
    ptype = (int**) malloc( size_ptype * sizeof(int*) );
    if( ptype == NULL ) error("generate_partition: not enough memory.");

    cnt = 0; cnt1 = 0;

    int data[p];


    combination(p, 0, data, ptype, &cnt);

    SubSet(arr, d-1, p-1, 0, data, 0, cp, &cnt1);//, fixed, fixed_cnt);

    for(int i=0;i<cnt;i++)
        for(int j=0;j<cnt1;j++)
        {
            par->length = p;
            par->cut_pnt = (int*) malloc( (p-1) * sizeof(int) );//new int[p-1];
            if( par->cut_pnt == NULL ) error("generate_partition: not enough memory.");
            par->prim_type = (int*) malloc( p * sizeof(int) );//new int[p];
            if( par->prim_type == NULL ) error("generate_partition: not enough memory.");
            for(int k=0;k<p;k++)
                par->prim_type[k] = ptype[i][k];
            for(int k=0;k<p-1;k++)
                par->cut_pnt[k] = cp[j][k];
            par->next = (parList*) malloc( sizeof(parList) );//new parList;
            prevpntr = par;
            par = par->next;
            (*parcnt)++;
        }

    free(cp);
    free(ptype);

    }


    free(par);
    prevpntr->next = NULL;
    par = strpntr;

}

/*----------------------------------------------------------------------------*/
/** Reset a given partition.
 */
void clear_partition( parList *par )
{
  /* check parameters */
  if( par == NULL )
      error("clear_partition: partition should be already malloc'ed.");

  if(par->next != NULL) clear_partition(par->next);

  //par->length = 0;
  //free(par->cut_pnt);
  //free(par->prim_type);
  //free(par->next);
  if(par->cut_pnt != NULL) free(par->cut_pnt);
  par->cut_pnt = NULL;
  if(par->prim_type != NULL) free(par->prim_type);
  par->prim_type = NULL;
  free(par);
  //par->next = NULL;
}
